function lcl = LCL(beta,bnet,data,cnode,enodes,cyclic,mixim,varargin)

% lcl = LCL(beta,bnet,data,cnode,enodes,cyclic,mixim)
%  Estimate the log conditional likelyhood.
%  If beta is empty, use bnet's conditional probabilities instead.
%  mixim is optional and should be used only for test purposes for now.
%  mixim is a matrix such that line i computes the sum of node i's parent
%  values.
%  If you prefer not to use mixim, do not provide it or set mixim=[].
%  Anyway, if beta is empty, mixim is ignored.
%  beta should be a column vector.

if length(beta)>0 % else use existing bnet.CPD - externalizing beta is a way to make that function compliant to gradient descent functions from NetLab
    if nargin>=7 && length(mixim)>0
        theta = beta2theta(mixim,beta);
    else
        theta = beta2theta(bnet,beta);
    end%if
    bnet = bnet_pak(bnet,theta); % the input beta is internalized
end%if

eps = 1e-6;

if nargin>7
    data = varargin{1};
end%if
[dim,n] = size(data);

lcl = 0;
if nargin<6 || ~islogical(cyclic) || cyclic
    engine = jtree_inf_engine(bnet);
else
    engine = pearl_inf_engine(bnet);
end%if

for i=1:n
    % for each evidence, compute the LCL (log(p_c_e))
    evidence = cell(1,dim);
    evidence(enodes) = num2cell(data(enodes,i));
    [aux_engine,LL] = enter_evidence(engine,evidence);
    marg = marginal_nodes(aux_engine,cnode,1);
    p_c_e = marg.T(data(cnode,i));
    if p_c_e < eps
        p_c_e = eps;
    end%if
    lcl = lcl + log(p_c_e); % global empirical LCL
end%for
lcl = lcl/n; % see equation 5 from the refered article

end%function