function glcl = LCLgrad(beta,bnet,data,cnode,enodes,cyclic,mixim,varargin)

% glcl = LCLgrad(beta,bnet,data,cnode,enodes,cyclic,mixim)
%  Estimate the gradient of the log conditional likelyhood as a column
%  vector. beta should be a column vector as well.
%  If beta is empty, use bnet's conditional probabilities instead.
%  mixim is optional and should be used only for test purposes for now.
%  mixim is a matrix such that line i computes the sum of node i's parent
%  values.
%  If you prefer not to use mixim, do not provide it or set mixim=[].
%  Anyway, if beta is empty, mixim is ignored.

if length(beta)==0
    theta = bnet_unpak(bnet);
elseif nargin>6 && length(mixim)>0
    theta = beta2theta(mixim,beta);
else
    theta = beta2theta(bnet,beta);
end%if
glcl = zeros(length(theta),1);
[dim,n] = size(data);

if nargin<6 || ~islogical(cyclic) || cyclic
    engine = jtree_inf_engine(bnet);
else
    engine = pearl_inf_engine(bnet);
end%if

for i=1:n
    
    P_f_e = [];
    P_df_e = [];
    P_f_ec = [];
    P_df_ec = [];

    evidence = cell(1,dim);

    evidence(enodes) = num2cell(data(enodes,i));
    [engine_e,LL] = enter_evidence(engine,evidence); % _e
    for dnode=1:dim
        P_f_e = [P_f_e; repmat( pa_infer(engine_e,bnet,dnode,false), bnet.node_sizes(dnode), 1 )]; % _f
        P_df_e = [P_df_e; pa_infer(engine_e,bnet,dnode,true)]; % _df
    end%for

    evidence{cnode} = data(cnode,i);
    [engine_ec,LL] = enter_evidence(engine,evidence); % _ec
    for dnode=1:dim
        P_f_ec = [P_f_ec; repmat( pa_infer(engine_ec,bnet,dnode,false), bnet.node_sizes(dnode), 1 )]; % _f
        P_df_ec = [P_df_ec; pa_infer(engine_ec,bnet,dnode,true)]; % _df
    end%for

    glcl = glcl + ( (P_df_ec - P_df_e) - theta .* (P_f_ec - P_f_e) );
    
end%for



function p_x_y  = pa_infer(engine_y,bnet,d,with_d)
% PA_INFER(ENGINE_Y, BNET, D, WITH_D) computes P_X_Y.
% For an evidence Y, infer joint probabilities for parents of D (index of a
% node in BNET), with or without D according to the WITH_D boolean.

pa_d = parents(bnet.dag,d);
if with_d % x = {d pa(d)}
    %[xvalues,pa_d] = node_parents_pair_values(bnet,d);
    x = [pa_d d];
else
    %[xvalues,pa_d] = parents_values(bnet,d);
    x = pa_d;
end%if
if length(x)>0
    marg = marginal_nodes(engine_y,x,1);
    p_x_y = marg.T(:);%element(marg.T,xvalues); % format output
else
    p_x_y = 0;
end%if

end%function


end%function