function mixing_matrix = bnet_mixim(bnet,duplicate)

% mixing_matrix = bnet_mixim(bnet,duplicate)
%  Return a sparse matrix that sums node values for each configuration of
%  parents of each node. The ordering of nodes is that of bnet_unpack.
%  If duplicate is provided and true, duplicate each line as many times as
%  the size of the corresponding child node, in order to get a square
%  matrix.

if nargin<2
    duplicate = false;
end%if

dim = length(bnet.CPD);
i = 0;
j = 0;
r = []; % row indices (see sparse)
c = []; % column indices
for d=1:dim
    % parents are expected to be first
    s = cpt_size(bnet,d);
    m = s(end); % == bnet.node_sizes(d)
    n = prod(s(1:end-1)); % n == #configurations for parents of d
    p = prod(s);
    if duplicate
        f = m;
    else
        f = 1;
    end%if
    if n==1 % no parents
        r = [r i+repmat(1:f,1,m)]; % j + [1 2 3 ... 1 2 3 ... ...]
        c = [c j+reshape(repmat(1:p,f,1),m*f,1)']; % j + [1 1 ... 2 2 ... 3 3 ... ...]
    else
        for k=1:n
            r = [r i+repmat(k:n:n*f,1,m)]; % j + [i i+n i+2n ... i i+n i+2n ... ...]
            c = [c j+reshape(repmat(k:n:p,f,1),m*f,1)']; % j + [i i ... i+n i+n ... i+2n i+2n ... ...]
        end%for
    end%if
    i = i + n*f;
    j = j + p;
end%for

mixing_matrix = sparse(r,c,ones(1,length(c)));

end%function